library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;
library UNISIM;
use UNISIM.VComponents.all;

entity TEST_CLOCK_ENABLE is
   generic(
      C_FL_WIDTH : integer := 132);
   port (
      ------------------  System Signals  ----------------------
      rst        : in std_logic;
      clk_100MHz : in std_logic;
      ECLK_25Mhz_o : out std_logic;
      
      ------------------  Control Signals  ----------------------
      ADD_i : in std_logic_vector(3 downto 0);
      SUB_i : in std_logic_vector(3 downto 0);
      
      ------------------  Outgoing Signals  ----------------------
      bram_fill_level_0_o : out std_logic_vector(C_FL_WIDTH-1 downto 0);
      bram_fill_level_1_o : out std_logic_vector(C_FL_WIDTH-1 downto 0);
      bram_fill_level_2_o : out std_logic_vector(C_FL_WIDTH-1 downto 0);
      bram_fill_level_3_o : out std_logic_vector(C_FL_WIDTH-1 downto 0)
      );
end TEST_CLOCK_ENABLE;

architecture Behavioral of TEST_CLOCK_ENABLE is

   signal RST_100MHz, ECLK_25MHz : std_logic; --synchronous reset.
   signal ADD_r : std_logic_vector(3 downto 0);
   signal SUB_r : std_logic_vector(3 downto 0);
   
   -- attribute keep : string;
   -- attribute keep of bram_fill_level_nw : signal is "true";
   attribute keep : string;
   attribute keep of RST_100MHz, ECLK_25MHz : signal is "true";
   attribute ASYNC_REG : string;
   attribute ASYNC_REG of ADD_r, SUB_r : signal is "true";

begin

   ADD_r <= ADD_i when rising_edge(clk_100MHz) and (ECLK_25MHz = '1');
   SUB_r <= SUB_i when rising_edge(clk_100MHz) and (ECLK_25MHz = '1');
   -- ECLK_25Mhz_o <= ECLK_25MHz;
   ECLK_25MHz_out : OBUF
   port map (O => ECLK_25MHz_o, 
             I => ECLK_25MHz); 
   
   RST_ECLK_INST: entity work.RST_ECLK
      port map (
         RST        => RST,
         CLK_100MHz => CLK_100MHz,
         RST_100MHz => RST_100MHz,
         ECLK_25MHz => ECLK_25MHz);

 
   FILL_LEVEL_0_INST: entity work.FILL_LEVEL
      generic map (
         C_FL_WIDTH => C_FL_WIDTH)
      port map (
         rst               => RST_100MHz,
         clk_100MHz        => clk_100MHz,
         eclk_25MHz        => eclk_25MHz,
         ADD_i             => ADD_r(0),
         SUB_i             => SUB_r(0),
         bram_fill_level_o => bram_fill_level_0_o);
 
   FILL_LEVEL_1_INST: entity work.FILL_LEVEL
      generic map (
         C_FL_WIDTH => C_FL_WIDTH)
      port map (
         rst               => RST_100MHz,
         clk_100MHz        => clk_100MHz,
         eclk_25MHz        => eclk_25MHz,
         ADD_i             => ADD_r(1),
         SUB_i             => SUB_r(1),
         bram_fill_level_o => bram_fill_level_1_o);

   FILL_LEVEL_2_INST: entity work.FILL_LEVEL
      generic map (
         C_FL_WIDTH => C_FL_WIDTH)
      port map (
         rst               => RST_100MHz,
         clk_100MHz        => clk_100MHz,
         eclk_25MHz        => eclk_25MHz,
         ADD_i             => ADD_r(2),
         SUB_i             => SUB_r(2),
         bram_fill_level_o => bram_fill_level_2_o);

   FILL_LEVEL_3_INST: entity work.FILL_LEVEL
      generic map (
         C_FL_WIDTH => C_FL_WIDTH)
      port map (
         rst               => RST_100MHz,
         clk_100MHz        => clk_100MHz,
         eclk_25MHz        => eclk_25MHz,
         ADD_i             => ADD_r(3),
         SUB_i             => SUB_r(3),
         bram_fill_level_o => bram_fill_level_3_o);

end Behavioral;