library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use ieee.numeric_std.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity RST_ECLK is
   port (RST        : in  std_logic;
         CLK_100MHz : in  std_logic;
         RST_100MHz : out std_logic;
         ECLK_25MHz : out std_logic
         );              
   attribute keep_hierarchy : string;
   attribute keep_hierarchy of RST_ECLK : entity is "YES";
end RST_ECLK;


architecture behav of RST_ECLK is

   ----------------------------------------------------------------------------
   -- constants  
   ----------------------------------------------------------------------------
   constant CLK_25MHz_VALUE_3_bc : std_logic_vector(1 downto 0) := "11";
   constant CLK_25MHz_VALUE_0_bc : std_logic_vector(1 downto 0) := "00";
   
   

   ----------------------------------------------------------------------------
   -- signals  
   ----------------------------------------------------------------------------
   signal rst_s : std_logic;
   signal CLK_25MHz_COUNT_s : std_logic_vector(1 downto 0);
   signal ECLK_25MHz_r, ECLK_25MHz_s : std_logic;

   attribute clock_signal : string;
   attribute clock_signal of CLK_100MHz  : signal is "yes";
   attribute period: string;
   attribute period of CLK_100MHz  : signal is "10 ns";
   attribute keep : string;
   attribute keep of rst_s : signal is "true";
   
begin
   rst_s <= rst when rising_edge(clk_100MHz);
   RST_100MHz <= rst_s;


-------------------------------------------------------------------------------
-- generate 25MHz
-------------------------------------------------------------------------------

   -------------------------------------------------------------------------------
   -- counter for 25 MHz clock signal
   -------------------------------------------------------------------------------
   CLK_25MHz_COUNTER : process (CLK_100MHz)
      -------------------------------------------------------------------------------
   begin
      if rising_edge (CLK_100MHz) then
         if (RST_s = '1') then
            CLK_25MHz_COUNT_s <= (others => '0');
         else
            if (CLK_25MHz_COUNT_s = CLK_25MHz_VALUE_0_bc) then
               CLK_25MHz_COUNT_s <= CLK_25MHz_VALUE_3_bc;
            else
               CLK_25MHz_COUNT_s <= CLK_25MHz_COUNT_s - '1';
            end if;
         end if;
      end if;
   end process;
   
   -------------------------------------------------------------------------------
   -- Generate 25 MHz clock enable pulse
   -------------------------------------------------------------------------------
   CLK_EN_25MHz_GEN : process (CLK_100MHz)
      -------------------------------------------------------------------------------
   begin
      if rising_edge (CLK_100MHz) then
         if (RST_s = '1') then
            ECLK_25MHz_r   <= '0';
         else
            if (CLK_25MHz_COUNT_s = CLK_25MHz_VALUE_0_bc) then
               ECLK_25MHz_r <= '1';
            else
               ECLK_25MHz_r <= '0';
            end if;
         end if;
      end if;
   end process;
   
   CLK_EN_25MHz_ShiftReg : process (CLK_100MHz)
      -------------------------------------------------------------------------------
   begin
      if rising_edge (CLK_100MHz) then
         if (RST_s = '1') then
            ECLK_25MHz_s <= '0';
         else
            ECLK_25MHz_s <= ECLK_25MHz_r;
         end if;
      end if;
   end process;

   -- ECLK_BUFG_INST : BUFG
      -- port map (I => ECLK_25MHz_sr0, O => ECLK_25MHz_s);

-------------------------------------------------------------------------------
-- connection to out port
-------------------------------------------------------------------------------      
   ECLK_25MHz <= ECLK_25MHz_s;
   
-------------------------------------------------------------------------------
end behav;
-------------------------------------------------------------------------------
